# -*- coding: utf-8 -*-
"""
Major Project Metadata.

This file should be used to get the version and other metadata whenever
technically possible, to reduce locations where this data has to be
maintained redundantly.
"""

version = '1.2.2'
author = u'Jyrki Pulliainen'
author_email = 'jyrki@dywypi.org'
url = 'https://github.com/spotify/dh-virtualenv'
